"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
const CicsSession_1 = require("../CicsSession");
const Program_definition_1 = require("./program/Program.definition");
const Transaction_definition_1 = require("./transaction/Transaction.definition");
const UrimapServer_definition_1 = require("./urimap-server/UrimapServer.definition");
const UrimapClient_definition_1 = require("./urimap-client/UrimapClient.definition");
const UrimapPipeline_definition_1 = require("./urimap-pipeline/UrimapPipeline.definition");
const Webservice_definition_1 = require("./webservice/Webservice.definition");
// Does not use the import in anticipation of some internationalization work to be done later.
const strings = require("../-strings-/en").default.DEFINE;
/**
 * Definition for the "define" group of commands under the CICS plugin
 */
const definition = {
    name: "define", aliases: ["def"],
    summary: strings.SUMMARY,
    description: strings.DESCRIPTION,
    type: "group",
    children: [Program_definition_1.ProgramDefinition,
        Transaction_definition_1.TransactionDefinition,
        UrimapServer_definition_1.UrimapServerDefinition,
        UrimapClient_definition_1.UrimapClientDefinition,
        UrimapPipeline_definition_1.UrimapPipelineDefinition,
        Webservice_definition_1.WebServiceDefinition],
    passOn: [
        {
            property: "options",
            value: CicsSession_1.CicsSession.CICS_CONNECTION_OPTIONS,
            merge: true,
            ignoreNodes: [
                { type: "group" }
            ]
        }
    ]
};
module.exports = definition;
//# sourceMappingURL=Define.definition.js.map